<?php
namespace Teknisa\EST\Modules\EstoqueCego\Controllers;

use Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;

class Est92000ConsultaInventarioCegoController extends TeknisaCrud {

    protected $dataSourceName = "inventestcego_paramfilial_paramgeral_filioper";
    protected $parameterBagColumns = ["CDFILIAL","DTINICIAL","DTFINAL","IDSITESTCEGO","CDOPERADOR"];

    protected $Est92000ConsultaInventarioCegoService;
    protected $dataSourceManager;
    protected $environment;
    
    public function __construct($crudService,$parameterBag, $Est92000Service,$dataSourceManager,$environment) {
        parent::__construct($crudService,$parameterBag,$environment);

        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->Est92000ConsultaInventarioCegoService = $Est92000Service;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $containerName = $request->getParameters()['origin']['containerName'];
            
            if($containerName === 'est92000_grid_relatorios'){
                $conditions = $request->getFilterCriteria()->getConditions();
                $newDataSet = [];
                $rows = array_filter($conditions, function($condition){ return $condition['columnName'] == "DATA"; });
                
                foreach($rows[0]['value'] as $row){
                    $allowGenerate = true;
                    $this->parameterBagColumns = ["CDFILIAL", "CDALMOXARIFE", "CDLOCALESTOQ", "DTCONTAGEM" ];
                    
                    if($allowGenerate){
                        $this->dataSourceName = 'iteminventcego_produto';
                    
                        $filterCriteria = $this->filterCriteriaBuild($row);
                        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
                        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
                        
                        $filialInfo = $this->Est92000ConsultaInventarioCegoService->getFilialInfo($row["CDFILIAL"]);
                        foreach($dataSet as &$dataSetRow){
                            $dataSetRow['DSFILIAL'] = $row["CDFILIAL"] ."  -  " . $filialInfo["NMFILIAL"];
                            $dataSetRow['DSDATA'] = $this->Est92000ConsultaInventarioCegoService->formatDSDATAField($row);
                            
                            $reportTypeLIC = array_filter($conditions, function($condition){return $condition['value'] == 'LIC';});
                            if(!empty($reportType)){
                                $dataSetRow['DSDATA'] = substr($row['DTCONTAGEM'],0,10);
                                $dataSetRow['QTINVENTESTCEGO_LIC'] =  "___________________";
                            }
                        }
                        
                        $newDataSet[] = $dataSet; 
                    }
                }
                $newDataSet = call_user_func_array('array_merge', $newDataSet);  //works like array flat in js
                $response->setDataSets(array(new DataSet("consultaInventarioCego", $newDataSet)));
            }else{
                $conditions = $request->getFilterCriteria()->getConditions();
                
                $this->Est92000ConsultaInventarioCegoService->formatMutipleFilialCode($conditions);
                $newFilterCriteria = new FilterCriteria($this->dataSourceName, $conditions);
                $filter = new DTO\Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                
                parent::find($filter, $response);
            }
        }catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    public function filterCriteriaBuild($row) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        foreach($this->parameterBagColumns as $fieldName){
            $filterCriteria->addCondition($fieldName, $row[$fieldName]);
        }
        return $filterCriteria;
    }
    
}
