<?php
namespace Teknisa\EST\Modules\Relatorios\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Ger09700ProjecaoDeEstoqueService {
    
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
		$this->connection = $entityManager->getConnection();
    }
    
    public function executaProcedureProjecaoDeEstoque(
        $P_FILIPROC,
        $P_CDFILIAL,
        $P_DTLEVINI,
        $P_DTLEVFIN,
        $P_PRODINI,
        $P_PRODFIN,
        $P_FILESTOQ,
        $P_IDGERSOL,
        $P_CONSSOL,
        $P_CONSAF,
        $P_CONSEST,
        $P_CONSROM,
        $P_FILIROM,
        $P_SERIROM,
        $P_OPERADOR,
        $P_TODOSPRODEST
    ) {
        $sql = "SELECT LPAD(TO_CHAR(TO_NUMBER(NVL(MAX(NRSEQESTPROJ), '0'))+1), 10, '0') P_NRSEQESTPROJ
                FROM TRAB_ESTPROJET
                WHERE CDFILESTPROJ = :P_CDFILESTPROJ";
        $params = array("P_CDFILESTPROJ"=>$P_FILIPROC);
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $P_NRSEQESTPROJ = $std->fetchAll();
        $P_NRSEQESTPROJ = $P_NRSEQESTPROJ[0]["P_NRSEQESTPROJ"];
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), "PROJETA_ESTOQUE");
        
        $procedure->addParam(new Param("P_FILIPROC", Param::PARAM_INPUT, $P_FILIPROC, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIAL", Param::PARAM_INPUT, $P_CDFILIAL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_DTLEVINI", Param::PARAM_INPUT, $P_DTLEVINI, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_DTLEVFIN", Param::PARAM_INPUT, $P_DTLEVFIN, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_PRODINI", Param::PARAM_INPUT, $P_PRODINI, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_PRODFIN", Param::PARAM_INPUT, $P_PRODFIN, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_FILESTOQ", Param::PARAM_INPUT, $P_FILESTOQ, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDGERSOL", Param::PARAM_INPUT, $P_IDGERSOL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CONSSOL", Param::PARAM_INPUT, $P_CONSSOL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CONSAF", Param::PARAM_INPUT, $P_CONSAF, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CONSEST", Param::PARAM_INPUT, $P_CONSEST, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CONSROM", Param::PARAM_INPUT, $P_CONSROM, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_FILIROM", Param::PARAM_INPUT, $P_FILIROM, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_SERIROM", Param::PARAM_INPUT, $P_SERIROM, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OPERADOR", Param::PARAM_INPUT, $P_OPERADOR, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_TODOSPRODEST", Param::PARAM_INPUT, $P_TODOSPRODEST, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NRSEQESTPROJ", Param::PARAM_OUTPUT, $P_NRSEQESTPROJ, Param::PARAM_TYPE_STR));
        
        $P_NRSEQESTPROJ_RETURN = $procedure->execute();
        $P_NRSEQESTPROJ_RETURN = $P_NRSEQESTPROJ_RETURN["P_NRSEQESTPROJ"];
        return $P_NRSEQESTPROJ_RETURN;
    }
    
    public function executaDeleteProjecaoDeEstoque($P_CDFILIESTPROJ, $P_NRSEQESTPROJ) {
        $sql = "DELETE FROM TRAB_ESTPROJET WHERE CDFILESTPROJ = :P_CDFILESTPROJ AND NRSEQESTPROJ = :P_NRSEQESTPROJ";
        $params = array(
            "P_CDFILESTPROJ"=>$P_CDFILIESTPROJ,
            "P_NRSEQESTPROJ"=>$P_NRSEQESTPROJ
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function executaProcedureProjecaoDeEstoqueDia($parameters) {
        $sql = "SELECT LPAD(TO_CHAR(TO_NUMBER(NVL(MAX(NRSEQESTPROJ), '0'))+1), 10, '0') P_NRSEQESTPROJ
                FROM TRAB_ESTPROJET
                WHERE CDFILESTPROJ = :P_CDFILESTPROJ";
        $params = array("P_CDFILESTPROJ"=> $parameters['P_FILIPROC']);
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $P_NRSEQESTPROJ = $std->fetchAll();
        $P_NRSEQESTPROJ = $P_NRSEQESTPROJ[0]["P_NRSEQESTPROJ"];
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), "PROJETA_ESTOQUE_DIA");
        
        $procedure->addParam(new Param("P_CDFILIAL", Param::PARAM_INPUT, $parameters['P_CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_DTLEVINI", Param::PARAM_INPUT, $parameters['P_DTLEVINI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_DTLEVFIN", Param::PARAM_INPUT, $parameters['P_DTLEVFIN'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_PRODINI", Param::PARAM_INPUT, $parameters['P_PRODINI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_PRODFIN", Param::PARAM_INPUT, $parameters['P_PRODFIN'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_ENTRINI", Param::PARAM_INPUT, $parameters['P_ENTRINI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_ENTRFIN", Param::PARAM_INPUT, $parameters['P_ENTRFIN'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_SAIDAINI", Param::PARAM_INPUT, $parameters['P_SAIDAINI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_SAIDAFIN", Param::PARAM_INPUT, $parameters['P_SAIDAFIN'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_TODOSPRODEST", Param::PARAM_INPUT, $parameters['TODOSPRODEST'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NRSEQESTPROJ", Param::PARAM_OUTPUT, $P_NRSEQESTPROJ, Param::PARAM_TYPE_STR));

        $result = $procedure->execute();
        return $result["P_NRSEQESTPROJ"];
    }
    
    public function getStockInfo($row) {
        $sql = "SELECT SUM(CALC_ESTOQ_CONS(:P_CDFILESTPROJ, :CDPROESTPROJ, TO_DATE(:P_DTPERLEVINI,'DD/MM/YYYY') + 1)) POSICAO
                  FROM PARAMGERAL";
        $params = array(
            "P_CDFILESTPROJ"=>$row['CDFILESTPROJ'],
            "CDPROESTPROJ"=>$row['CDPROESTPROJ'],
            "P_DTPERLEVINI"=>$row['DTESTOQUE']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch()['POSICAO'];
    }
    
    public function getReserveInfo($row) {
        $sql = "SELECT SUM(QTRESERVESTOL) QTDE
                  FROM RESERVAPEDEXOL, RELACPRODOL
                 WHERE CDFILRESOL = :P_CDFILESTPROJ
                   AND DTRESEMBARQ = :P_DTPERLEVINI
                   AND RELACPRODOL.CDPRARVORIG = RESERVAPEDEXOL.CDPRODUTO
                   AND RELACPRODOL.CDMARCA     = RESERVAPEDEXOL.CDMARCA
                   AND RELACPRODOL.CDPRODARVOL = :CDPRODUTO";
        $params = array(
            "P_CDFILESTPROJ"=>$row['CDFILESTPROJ'],
            "P_DTPERLEVINI"=>$row['DTESTOQUE'],
            "CDPRODUTO"=>$row['CDPRODUTO']
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch()['QTDE'];
    }
    
    public function formatMutipleFields($values){
        $result = '_';
        foreach($values as $value) {
            $result .= $value . '_';
        }
        return $result;
    }
}