<?php
namespace Fiscal\Logic\ImportaGeneroProd;

use Geral\Entities\GeneroProd;
use Geral\Util\DateUtil;
use Geral\Util\Ambiente;
use Indeterminado\Entities\RelacProdItGen;
use Setforms2\Shared\Registry;

class GeneroProduto
{
    public static function replicaGeneroProd($nrorg, $cdLivroOrig, $cdLivroDest) {

        /*Validação para verificar se o cadastro de gênero do produto do livro de origem está vazio.*/
        self::validaGeneroProdLivroVazio($nrorg, $cdLivroOrig);

        /*Valida se existe algum cadastro de Produto por Tp. Item/Gênero pois assim os tipos de itens atuais não podem ser substituídos.*/
        $proditgen = RelacProdItGen::findByNrorgAndCdlivromag($nrorg, $cdLivroDest);
        if ($proditgen == null) {
            $connection = Registry::getEntityManagerConnection();
            $connection->beginTransaction();

            try {
                self::replicaRegistrosGenero($nrorg, $cdLivroOrig, $cdLivroDest);
                $connection->commit();
            } catch (\Exception $e) {
                $connection->rollback();
                throw $e;
            }

        } else {
            throw new \Exception ("Operação Bloqueada! Os cadastros não podem ser substituídos pois estão sendo usados na aba Produto por Tipo de Item/Gênero.");
        }
    }

    public static function validaGeneroProdLivroVazio($nrorg, $cdLivroMag)
    {
        /*A função valida se existe algum gênero do produto cadastrado para o livro informado como parãmetro.*/
        $generoprod = GeneroProd::findOneByNrorgAndCdlivromag($nrorg, $cdLivroMag);

        if ($generoprod == null) {
            throw new \Exception ("Operação Bloqueada! Não existe gênero do produto cadastrado para o livro magnético de origem informado.");
        }
    }

    public static function replicaRegistrosGenero($nrorg, $cdLivroOrig, $cdLivroDest)
    {
        /*A função replica os gêneros do produto entre os livros conforme os parâmetros informados, de forma que,
          são excluídos todos os gêneros do livro de destino e então são inseridos todos os cadastros do livro
          de origem para o livro de destino.*/

        $dataAtual = DateUtil::getDataAtual();
        $cdoperador = Ambiente::getCdOperador();

        try {

            /*São apagados os cadastros de Gênero do produto do livro de destino para serem incluídos os novos.*/
            $repositorioGeneroProd = GeneroProd::getRepository();
            $repositorioGeneroProd->deleteByCdlivromagAndNrorg($cdLivroDest, $nrorg);
            Registry::getEntityManager()->flush();

            /*São buscados todos os generos do produto do livro de origem e para cada um são criados novos itens com o código do livro de destino.*/
            $generos = GeneroProd::findByNrorgAndCdlivromag($nrorg, $cdLivroOrig);

            foreach ($generos as $genero) {
                $newgenero = new GeneroProd();
                $newgenero->setCdlivromag($cdLivroDest);
                $newgenero->setCdgenprod($genero->getCdgenprod());
                $newgenero->setDsgenprod($genero->getDsgenprod());
                $newgenero->setNrorginclusao($nrorg);
                $newgenero->setCdoperultatu($cdoperador);
                $newgenero->setNrorg($nrorg);
                $newgenero->setDtinclusao($dataAtual);
                $newgenero->setDtultatu($dataAtual);
                $newgenero->setNrorgultatu($nrorg);
                $newgenero->setCdoperinclusao($cdoperador);

                /*Um a um os gêneros são persistidos. O flush é executado na função de origem 'replicaGeneroProd'.*/
                $newgenero->persist();
            }

            Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }

    }

}