<?php
/**
 * Namespace Planejamento\Logic\Cardapio
 *
 * PHP version 5
 *
 * @category Php
 * @package  Planejamento\Logic\Cardapio
 * @author   matheussouza <matheus.souza@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Planejamento\Logic\Cardapio;

use Geral\Entities\Filial;
use Geral\Entities\Lista;
use Geral\Entities\Paramgeral;
use Geral\Entities\Servico;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Indeterminado\Entities\Logexclcardapio;
use Planejamento\Entities\Cardadia;
use Planejamento\Entities\Compcard;
use Planejamento\Entities\Compcarddel;
use Planejamento\Entities\Ctrlaprovcard;

/**
 * Class ExclusaoCardapio
 *
 * @category Php
 * @package  Planejamento\Logic\Cardapio
 * @author   matheussouza <matheus.souza@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class ExclusaoCardapio
{
    /**
     * Realiza a exclusão dos cardápios
     *
     * @param $nrlista
     * @param $dtinicial
     * @param $dtfinal
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiCardapios($nrlista, $dtinicial, $dtfinal)
    {
        try {
            $nrorg        = Ambiente::getNrorg();
            $servicos     = Servico::getRepository()->getServicosDescentralizados(
                $nrorg, $nrlista
            );
            $paramgeral   = Paramgeral::getObjectByWhere(
                'NRORG = ? AND CDPARGERAL = ?', array($nrorg, '0001')
            );
            $idutcardpadr = $paramgeral->getIdutcardpadr();

            if (empty($idutcardpadr)) {
                $idutcardpadr = 'N';
            }

            if ($servicos && $idutcardpadr == 'S') {
                $idutcardpadr = 'D';
            }

            $existecardapio = Cardadia::getRepository()->getNrcardapio(
                $nrorg, $nrlista, $dtinicial, $dtfinal
            );

            if (empty($existecardapio)) {
                tk_abort(tk_get_message_by_name('msg_pla00128_semCardapio'));
            }

            switch ($idutcardpadr) {
            case 'N':
                $cardapio     = Cardadia::getRepository()->getCardapioLivre(
                    $nrorg, $nrlista, $dtinicial, $dtfinal
                );
                $cardapiobloq = Cardadia::getRepository()->getCardapioBloqLivre(
                    $nrorg, $nrlista, $dtinicial, $dtfinal
                );
                break;
            case 'S':
                $cardapio     = Cardadia::getRepository()->getCardapioPadronizado(
                    $nrorg, $nrlista, $dtinicial, $dtfinal
                );
                $cardapiobloq = Cardadia::getRepository()
                    ->getCardapioBloqPadronizado(
                        $nrorg, $nrlista, $dtinicial, $dtfinal
                    );
                break;
            case 'D':
                $cardapio     = Cardadia::getRepository()
                    ->getCardapioDescentralizado(
                        $nrorg, $nrlista, $dtinicial, $dtfinal
                    );
                $cardapiobloq = Cardadia::getRepository()
                    ->getCardapioBloqDescentralizado(
                        $nrorg, $nrlista, $dtinicial, $dtfinal
                    );
                break;
            }

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            foreach ($cardapio as $card) {
                $filial     = $card['CDFILIAL'];
                $servico    = $card['CDSERVFILI'];
                $nrcardapio = $card['NRCARDAPIO'];
                $dtcardapio = DateUtil::truncateString($card['DTCARDAPIO']);
                $dtcardapio = DateUtil::getDataDeString($dtcardapio);
                if (self::_dependeLiberacao($filial, $servico, $idutcardpadr)) {
                    $dtmescardaprov = DateUtil::getPrimeiroDiaDoMes($dtcardapio);
                    $dtmescardaprov = $dtmescardaprov->format('d/m/Y');
                    $ctrlaprovcard  = Ctrlaprovcard::getObjectByWhere(
                        'NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ? AND
                        DTMESCARDAPROV = ? AND DTLIBECARDMATR IS NOT NULL',
                        array(
                            $nrorg, $filial, $servico, $dtmescardaprov
                        )
                    );

                    if ($ctrlaprovcard) {
                        $idpratoespe = 'TODOS';
                    } else {
                        $idpratoespe = 'N';
                    }

                    if ($idpratoespe != 'N') {
                        $compcard = Compcard::getObjectsByWhere(
                            'NRORG = ? AND IDPRATOESPE <> ? AND CDFILIAL = ? AND
                            NRCARDAPIO = ?',
                            array($nrorg, $idpratoespe, $filial, $nrcardapio)
                        );
                        foreach ($compcard as $comp) {
                            $cdfilauxprat = $comp->getCdfilauxpratmat();
                            $cdprato      = $comp->getCdpratomat();
                            if (empty($cdfilauxprat) && empty($cdprato)) {
                                $cdfilauxprat = $comp->getCdfilauxprat();
                                $cdprato      = $comp->getCdprato();
                            }
                            Compcarddel::insere(
                                $nrorg, $filial, $servico, $dtcardapio,
                                $comp->getNrseqpratcar(), 'M', $cdfilauxprat,
                                $cdprato
                            );
                        }
                    }
                }

                if ($idutcardpadr == 'S') {
                    $fili    = Filial::getObjectByWhere(
                        'NRORG = ? AND CDFILIAL = ?', array($nrorg, $filial)
                    );
                    $central = $fili->getCdfilcentpme();
                    if (empty($central)) {
                        $compcard = Compcard::getObjectsByWhere(
                            'NRORG = ? AND CDFILIAL = ? AND NRCARDAPIO = ?',
                            array($nrorg, $filial, $nrcardapio)
                        );
                        foreach ($compcard as $comp) {
                            $compcarddel = Compcarddel::getObjectByWhere(
                                'NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ? AND
                                DTPRATEXCL = ? AND NRSEQPRATEXCL = ?',
                                array(
                                    $nrorg, $filial, $servico,
                                    $dtcardapio->format('d/m/Y'),
                                    $comp->getNrseqpratcar()
                                )
                            );
                            if (empty($compcarddel)) {
                                Compcarddel::insere(
                                    $nrorg, $filial, $servico, $dtcardapio,
                                    $comp->getNrseqpratcar(), 'M',
                                    $comp->getCdfilauxpratmat(),
                                    $comp->getCdpratomat()
                                );
                            } else {
                                Compcarddel::atualiza(
                                    $nrorg, $filial, $servico,
                                    $dtcardapio->format('d/m/Y'),
                                    $comp->getNrseqpratcar(), 'M'
                                );
                            }
                        }
                    }
                }

                Logexclcardapio::insere(
                    $nrorg, $filial, $servico, $dtcardapio,
                    DateUtil::getDataDeString($dtinicial),
                    DateUtil::getDataDeString($dtfinal)
                );

                Compcard::deleta($nrorg, $filial, $nrcardapio);
                $connection->flush();

                Cardadia::deleta($nrorg, $filial, $nrcardapio);
                $connection->flush();

            }

            if ($cardapiobloq) {
                tk_message(
                    tk_get_message_by_name('msg_pla00128_cardapioBloqueado'),
                    'warning'
                );
            } else {
                tk_message(
                    tk_get_message_by_name('msg_pla00128_cardapioExcluido'),
                    'success'
                );
            }

            /** @var \Geral\Repositories\Lista $entidadeLista */
            $entidadeLista = Lista::getRepository();
            $entidadeLista->deletaRegistroLista($nrorg, $nrlista);
            $connection->flush();

            $connection->commit();
//            Lista::deleta($nrorg, $nrlista);
        } catch (\Exception $e) {
            $connection->rollback();
//            Lista::deleta($nrorg, $nrlista);
            if (strstr($e->getMessage(), 'FKPLA00442CARDADIA') !== false) {
                tk_warning(tk_get_message_by_name('msg_pla00118_haDistruibCardapio', array($dtcardapio->format('d/m/Y'))));
                return false;
            } else {
                tk_warning($e->getMessage());
            }
        }
    }

    /**
     * Verifica se o serviço depende da liberação (é descentralizado)
     *
     * @param $filial
     * @param $servico
     * @param $tipopadrao
     *
     * @return bool
     */
    private static function _dependeLiberacao($filial, $servico, $tipopadrao)
    {
        if ($tipopadrao == 'D') {
            $nrorg          = Ambiente::getNrorg();
            $servico        = Servico::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ?',
                array($nrorg, $filial, $servico)
            );
            $idcarddescentr = $servico->getIdcarddescentr();
            if (empty($idcarddescentr)) {
                $idcarddescentr = 'N';
            }
            $retorno = ($idcarddescentr == 'S');
        } else {
            $retorno = false;
        }
        return $retorno;
    }
}