/*---------------------------------------------------------+
|							   |
|         	 Procedure: ETL_FINANCEIROGER		   |
|                                               	   |
|          Rotina para insero de dados de BI - 	   |
|                Contas a pagar/pagas e 		   |
|	       Contas a receber/recebidas.      	   |
|                                                          |
|                 Tabelas Preenchidas:               	   |
|	                                                   |
|          1:   ETLCONTAPAG - Pagamento                    |
|          2:   ETLCONTAREC - Recebimento          	   |
|                                               	   |
|          Criao:                                  	   |
|          04/03/2008 - Luiz Passos                        |
|						           |	   
|          Modificao:			                   |
|          28/04/2008 - Leandro Corra Gonalves           |
|      						           |
|      *Correo de Valores validados de acordo com os     |
|            Relatrios PAG11001 E REC11001                |
|	                                                   |           
+----------------------------------------------------------*/	                                               			 
                                                            
create or replace
PROCEDURE ETL_FINANCEIROGER
                                    (P_DTINI      IN DATE,
                                     P_DTFINA     IN DATE)
   IS
   V_DTFIM           DATE;
   V_DTINI           DATE;
   V_DTMOV           VARCHAR2(10);

   V_MES             VARCHAR2(02);
   V_ANO             VARCHAR2(04);
   V_BIM             VARCHAR2(06);
   V_TRI             VARCHAR2(06);
   V_SEM             VARCHAR2(06);

   V_COUNTER       NUMBER       NOT NULL := 0;
BEGIN

  V_MES  :=  TO_CHAR(P_DTINI,'MM');
  V_ANO  :=  TO_CHAR(P_DTINI,'YYYY');

  DELETE FROM ETLCONTAPAG
  WHERE (ANETLPAG > V_ANO)
    OR (ANETLPAG = V_ANO AND MEETLPAG >= V_MES);
  COMMIT;

  DELETE FROM ETLCONTAREC
  WHERE (ANETLREC > V_ANO)
    OR (ANETLREC = V_ANO AND MEETLREC >= V_MES);
  COMMIT;

  V_DTINI := TO_DATE('01'||TO_CHAR(P_DTINI,'MMYYYY'),'DDMMYYYY');

  WHILE (V_DTINI  <=  P_DTFINA) LOOP
    V_DTFIM := LAST_DAY( V_DTINI);

    FOR C_ETL IN
      (SELECT F.CDFILIAL, F.DTINICIALBI,  F.CDEMPRESA
       FROM  FILIAL F
       WHERE F.IDFUNCAOFILI IS NULL    --Desconsidera Regionais e Operadores Logsticos
       AND F.DTINICIALBI IS NOT NULL --Desconsidera Filiais Sem Data de BI cadastrada
      )
    LOOP

      IF TO_DATE('01'||TO_CHAR(C_ETL.DTINICIALBI,'MMYYYY'),'DDMMYYYY') <= V_DTINI THEN  --Incio da condio por data de BI
    
      V_MES := TO_CHAR(V_DTFIM,'MM');
      V_ANO := TO_CHAR(V_DTFIM,'YYYY');
      V_BIM := TO_CHAR(TRUNC((V_MES - 1) / 2) + 1)||' Bim';
      V_TRI := TO_CHAR(TRUNC((V_MES - 1) / 3) + 1)||' Tri';
      V_SEM := TO_CHAR(TRUNC((V_MES - 1) / 6) + 1)||' Sem';

      FOR C_PAGAR IN
         (SELECT CDFILIAL,
                 CDFORNECED,
                 CDEMPRESA,
                 CDTPCTPAGAR,
                 CDCLASSFINA,
                 DTENTRSAID,
                 SUM(VRORIGINAL) AS VRORIG,
                 SUM(VRAVENCER) AS VRAVENCER,
                 SUM(VRVENCIDAS) AS VRVENCIDAS,
                 SUM(VRREAIS) AS VRREAL,
                 SUM(VRPREV) AS VRPREVISTA,
                 SUM(VRACRESCIMO) AS VRACRESCIMO,
                 SUM(VRDESCONTO) AS VRDESCONTO,
                 SUM(VRCTABERTO) AS VRCTABERTO
            FROM (SELECT CP.CDFILIAL,
                        NF.CDFORNECED,
                        CP.CDEMPRESA,
                        CP.CDTPCTPAGAR,
                        CP.CDCLASSFINA,
                        NF.DTENTRSAID,
                        SUM(NVL(CP.VRORIGPAG,0)) AS VRORIGINAL,
                        SUM(NVL(CP.VRATUAPAG,0)) AS VREFETIVPAG,
                        SUM(NVL(CASE WHEN TRUNC(CP.DTATUAVENPAG)  >= TRUNC(SYSDATE) THEN CP.VRATUAPAG END,0)) AS VRAVENCER,
                        SUM(NVL(CASE WHEN TRUNC(CP.DTATUAVENPAG)  < TRUNC(SYSDATE) THEN CP.VRATUAPAG END,0)) AS VRVENCIDAS,
                        SUM(DECODE(CP.IDSTATPAG, 'B', CP.VRATUAPAG, 0)) AS VRREAIS,
                        SUM(DECODE(NF.IDSTATUS, 'V',CP.VRATUAPAG, 0)) AS VRPREV,
                        SUM(DECODE(TA.IDAJUCTPAG,'A', AJ.VRAJUSTEPAG,0)) AS VRACRESCIMO,
                        SUM(DECODE(TA.IDAJUCTPAG,'D', AJ.VRAJUSTEPAG,0)) AS VRDESCONTO,
                        SUM(DECODE(CP.IDSTATPAG, 'A', CP.VRATUAPAG, 0))  AS VRCTABERTO
                    FROM CONTAPAGAR CP, NOTAFISCAL NF, AJCTPAGAR AJ,
                         TIPOAJPAGAR TA, TIPOOPERAC OP
                   WHERE CP.CDFILIAL = NF.CDFILIAL
                     AND CP.NRLANCTONF = NF.NRLANCTONF
                     AND TRUNC(CP.DTATUAVENPAG) BETWEEN TRUNC(V_DTINI) AND TRUNC(V_DTFIM)
                     AND CP.CDFILIAL = C_ETL.CDFILIAL
                     AND CP.CDFILIAL = AJ.CDFILIAL(+)
                     AND CP.NRLANCTONF = AJ.NRLANCTONF(+)
                     AND CP.NRSEQVENPAG = AJ.NRSEQVENPAG(+)
                     AND CP.NRPARCELAPAG = AJ.NRPARCELAPAG(+)
                     AND AJ.CDAJUCTPAG = TA.CDAJUCTPAG(+)
                     AND CP.CDEMPRESA IS NOT NULL
                     AND CP.VRATUAPAG > 0
                     AND NOT(NF.IDSTATUS IN('C','G','P','R'))
                     AND OP.CDTIPOOPER = NF.CDTIPOOPER
                     AND OP.IDENTRSAIDOP = 'E'

                   GROUP BY CP.CDFILIAL, NF.CDFORNECED, CP.CDEMPRESA, CP.CDTPCTPAGAR, CP.CDCLASSFINA, NF.DTENTRSAID)
            GROUP BY CDFILIAL, CDFORNECED, CDEMPRESA, CDTPCTPAGAR, CDCLASSFINA, DTENTRSAID)
        LOOP
           V_DTMOV := TO_CHAR(C_PAGAR.DTENTRSAID, 'DD/MM/YYYY');
           INSERT INTO ETLCONTAPAG
             (ANETLPAG,   MEETLPAG,    BIETLPAG,     TRETLPAG,    SEETLPAG,
              CDFILIAL,   CDFORNECED,  CDEMPRESA,    CDTPCTPAGAR , CDCLASSFINA, DTETLMOVPAG,
              VRETLORIGPAG,VRETLPAGO,  VRETLAPAGAR, VRETLPAGACRE, VRETLPAGDESC, VRETLPAGVENCIDA,
              VRETLPAGVENCER,VRETLPAGPREV)
           VALUES
             (V_ANO, V_MES, V_BIM, V_TRI, V_SEM,
              C_ETL.CDFILIAL, C_PAGAR.CDFORNECED, C_PAGAR.CDEMPRESA, C_PAGAR.CDTPCTPAGAR, C_PAGAR.CDCLASSFINA, V_DTMOV,
              C_PAGAR.VRORIG, C_PAGAR.VRREAL, C_PAGAR.VRCTABERTO, C_PAGAR.VRACRESCIMO, C_PAGAR.VRDESCONTO,
              C_PAGAR.VRVENCIDAS, C_PAGAR.VRAVENCER, C_PAGAR.VRPREVISTA);

           V_COUNTER := V_COUNTER+1;
           IF V_COUNTER >= 500 THEN
             COMMIT;
             V_COUNTER := 0;
           END IF;
        END LOOP;

       V_COUNTER := 0;
       
       FOR C_RECEB IN
         (SELECT CDFILIAL,
                 CDCLIENTE,
                 CDEMPRESA,
                 CDTPCTRECEB,
                 CDCLASSFINA,
                 DTENTRSAID,
                 SUM(VRORIGINAL) AS VRORIG,
                 SUM(VRAVENCER) AS VRAVENCER,
                 SUM(VRVENCIDAS) AS VRVENCIDAS,
                 SUM(VRREAIS) AS VRREAL,
                 SUM(VRPREV) AS VRPREVISTA,
                 SUM(VRACRESCIMO) AS VRACRESCIMO,
                 SUM(VRDESCONTO) AS VRDESCONTO,
                 SUM(VRCTABERTO) AS VRCTABERTO
            FROM (SELECT CR.CDFILIAL,
                        NF.CDCLIENTE,
                        CR.CDEMPRESA,
                        CR.CDTPCTRECEB,
                        CR.CDCLASSFINA,
                        NF.DTENTRSAID,
                        SUM(NVL(CR.VRORIGREC,0)) AS VRORIGINAL,
                        SUM(NVL(CR.VRATUAREC,0)) AS VREFETIVREC,
                        SUM(NVL(CASE WHEN TRUNC(CR.DTATUAVENREC)  >= TRUNC(SYSDATE) THEN CR.VRATUAREC END,0)) AS VRAVENCER,
                        SUM(NVL(CASE WHEN TRUNC(CR.DTATUAVENREC)  < TRUNC(SYSDATE) THEN CR.VRATUAREC END,0)) AS VRVENCIDAS,
                        SUM(DECODE(CR.IDSTATREC, 'B', CR.VRATUAREC, 0)) AS VRREAIS,
                        SUM(DECODE(NF.IDSTATUS, 'V',CR.VRATUAREC, 0)) AS VRPREV,
                        SUM(DECODE(TA.IDAJUCTREC,'A', AJ.VRAJUSTEREC,0)) AS VRACRESCIMO,
                        SUM(DECODE(TA.IDAJUCTREC,'D', AJ.VRAJUSTEREC,0)) AS VRDESCONTO,
                        SUM(DECODE(CR.IDSTATREC, 'A', CR.VRATUAREC, 0))  AS VRCTABERTO
                    FROM CONTARECEBER CR, NOTAFISCAL NF, AJCTRECEBER AJ,
                         TIPOAJRECEBER TA, TIPOOPERAC OP  
                   WHERE CR.CDFILIAL = NF.CDFILIAL
                     AND CR.NRLANCTONF = NF.NRLANCTONF
                     AND TRUNC(CR.DTATUAVENREC) BETWEEN TRUNC(V_DTINI) AND TRUNC(V_DTFIM)
                     AND CR.CDFILIAL = C_ETL.CDFILIAL
                     AND CR.CDFILIAL = AJ.CDFILIAL(+)
                     AND CR.NRLANCTONF = AJ.NRLANCTONF(+)
                     AND CR.NRSEQVENREC = AJ.NRSEQVENREC(+)
                     AND CR.NRPARCELAREC = AJ.NRPARCELAREC(+)
                     AND AJ.CDAJUCTREC = TA.CDAJUCTREC(+)
                     AND CR.CDEMPRESA IS NOT NULL
                     AND CR.VRATUAREC > 0
                     AND NOT(NF.IDSTATUS IN('C','G','P','R'))
                     AND OP.CDTIPOOPER = NF.CDTIPOOPER
                     AND OP.IDENTRSAIDOP = 'E'
                  
                   GROUP BY CR.CDFILIAL, NF.CDCLIENTE, CR.CDEMPRESA, CR.CDTPCTRECEB, CR.CDCLASSFINA, NF.DTENTRSAID)
            GROUP BY CDFILIAL, CDCLIENTE, CDEMPRESA, CDTPCTRECEB, CDCLASSFINA, DTENTRSAID)
      
          LOOP
             V_DTMOV := TO_CHAR(C_RECEB.DTENTRSAID, 'DD/MM/YYYY');
             INSERT INTO ETLCONTAREC
               (ANETLREC,   MEETLREC,    BIETLREC,     TRETLREC,    SEETLREC,
                CDFILIAL,   CDCLIENTE,  CDEMPRESA,    CDTPCTRECEB , CDCLASSFINA, DTETLMOVREC,
                VRETLORIGREC,VRETLRECEB,  VRETLARECEB, VRETLRECACRE, VRETLRECDESC, VRETLRECVENCIDA,
                VRETLRECVENCER,VRETLRECPREV)
             VALUES
               (V_ANO, V_MES, V_BIM, V_TRI, V_SEM,
                C_ETL.CDFILIAL, C_RECEB.CDCLIENTE, C_RECEB.CDEMPRESA, C_RECEB.CDTPCTRECEB, C_RECEB.CDCLASSFINA, V_DTMOV,
                C_RECEB.VRORIG,C_RECEB.VRREAL, C_RECEB.VRCTABERTO, C_RECEB.VRACRESCIMO, C_RECEB.VRDESCONTO,
                C_RECEB.VRVENCIDAS, C_RECEB.VRAVENCER, C_RECEB.VRPREVISTA);

             V_COUNTER:= V_COUNTER+1;
             IF V_COUNTER >= 500 THEN
               COMMIT;
               V_COUNTER := 0;
             END IF;
         END LOOP;
      END IF;
    END LOOP;
    COMMIT;
    V_DTINI := ADD_MONTHS(V_DTINI,1);
  END LOOP;
END;
/

/*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   EVENTOAEV 
    WHERE  NMOBJBDEVEN = 'ETL_FINANCEIROGER';
    
    IF V_CDSEQUENCIAL = 0 THEN     --> S ADICIONA A PROCEDURE SE ELA NO EXISTIR AINDA NO CADASTRO
        BEGIN
            SELECT LPAD(NVL(MAX(LPAD(CDEVENTOAEV,10,'0'))+1,0),10,'0')
            INTO   V_CDSEQUENCIAL
            FROM   EVENTOAEV;
        
            INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
            VALUES
            (V_CDSEQUENCIAL,'Contas do Financeiro','STP',NULL,SYSDATE,'ETL_FINANCEIROGER');
            
            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTINI','D','E','Data Inicial','N',0,'01');
            
            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTFINA','D','E','Data Final','N',0,'02');
        
            COMMIT;
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001,'Falha no cadastro de eventos agendveis.');
        END;
    END IF;
END;
/

/*REGISTRANDO FATOS E DIMENSES*/

BEGIN                                       

  /*ETL DE CONTAS A PAGAR/PAGAS*/
   CRIA_FATOS('ETLCONTAPAG', 'VRETLORIGPAG',  'Vr. original (contas pagas)',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLPAGO',     'Vr efetivamente pago',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLAPAGAR',   'Vr. contas aberto (pagas)',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLPAGACRE',  'Acrscimos contas pagas',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLPAGDESC',  'Descontos contas pagas',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLPAGVENCIDA','Vr. de contas vencidas(PAG)',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLPAGVENCER',  'Vr. de contas a vencer(PAG)',  'Financeiro');
   CRIA_FATOS('ETLCONTAPAG', 'VRETLPAGPREV',  'Vr. de contas previstas(PAG)',  'Financeiro');

   CRIA_DIME('ETLCONTAPAG',  'CDFILIAL',     'Filial',              'FILIAL',     'CDFILIAL',     'NMFILIAL',     NULL);
   CRIA_DIME('ETLCONTAPAG',  'CDEMPRESA',    'Empresa',             'EMPRESA',    'CDEMPRESA',    'NMEMPRESA',    NULL);
   CRIA_DIME('ETLCONTAPAG',  'CDTPCTPAGAR',  'Tipo de Conta Paga',       'TIPOCTPAGAR','CDTPCTPAGAR',  'DSTPCTPAGAR',  NULL);
   CRIA_DIME('ETLCONTAPAG',  'CDFORNECED',   'Fornecedor',          'FORNECEDOR', 'CDFORNECED',   'NMFANTFORN',   NULL);
   CRIA_DIME('ETLCONTAPAG',  'CDCLASSFINA',  'Classificao Financeira','CLASSFINANC', 'CDCLASSFINA','DSCLASSFINA',   NULL);
   CRIA_DIME('ETLCONTAPAG',  'ANETLPAG',     'Ano',                  NULL,         NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAPAG',  'MEETLPAG',     'Ms',                  NULL,         NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAPAG',  'BIETLPAG',     'Bimestre',             NULL,         NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAPAG',  'TRETLPAG',     'Trimestre',            NULL,         NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAPAG',  'SEETLPAG',     'Semestre',             NULL,         NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAPAG',  'DTETLMOVPAG',  'Data da Movimentao', NULL,         NULL,           NULL,          NULL);

   /*ETL DE CONTAS A RECEBER/RECEBIDAS*/
   CRIA_FATOS('ETLCONTAREC', 'VRETLORIGREC', 'Vr. original (contas receb.)',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLRECEB',   'Vr. efetivamente recebido',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLARECEB',  'Vr. contas aberto (receb.)',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLRECACRE', 'Acrscimos contas recebidas',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLRECDESC', 'Descontos contas recebidas',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLRECVENCIDA','Vr. contas vencidas(REC)',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLRECVENCER',  'Vr. contas a vencer(REC)',  'Financeiro');
   CRIA_FATOS('ETLCONTAREC', 'VRETLRECPREV',  'Vr. Contas previstas(REC)',  'Financeiro');



   CRIA_DIME('ETLCONTAREC',  'CDFILIAL',     'Filial',              'FILIAL',       'CDFILIAL',     'NMFILIAL',     NULL);
   CRIA_DIME('ETLCONTAREC',  'CDEMPRESA',    'Empresa',             'EMPRESA',      'CDEMPRESA',    'NMEMPRESA',    NULL);
   CRIA_DIME('ETLCONTAREC',  'CDTPCTRECEB',  'Tipo de Conta Recebida','TIPOCTRECEBER','CDTPCTRECEB','DSTPCTRECEB',  NULL);
   CRIA_DIME('ETLCONTAREC',  'CDCLASSFINA',  'Classificacao Financeira','CLASSFINANC', 'CDCLASSFINA','DSCLASSFINA',NULL);
   CRIA_DIME('ETLCONTAREC',  'CDCLIENTE',    'Cliente',             'CLIENTE',      'CDCLIENTE',    'NMFANTCLIE',   NULL);
   CRIA_DIME('ETLCONTAREC',  'ANETLREC',     'Ano',                  NULL,           NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAREC',  'MEETLREC',     'Ms',                  NULL,           NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAREC',  'BIETLREC',     'Bimestre',             NULL,           NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAREC',  'TRETLREC',     'Trimestre',            NULL,           NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAREC',  'SEETLREC',     'Semestre',             NULL,           NULL,           NULL,          NULL);
   CRIA_DIME('ETLCONTAREC',  'DTETLMOVREC',  'Data da Movimentao', NULL,         NULL,           NULL,          NULL);
   COMMIT;
END;
/

-- VERSAO --
BEGIN
   ATUVEROBJ('ETL_FINANCEIROGER','5.05.020');
END;
/







